/*
 * This file is part of essential.
 *
 *     essential is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     essential is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with essential.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.craftforge.essential.context.guice;

import com.google.inject.Guice;
import com.google.inject.Injector;
import com.google.inject.Module;
import com.google.inject.Stage;
import com.google.inject.servlet.GuiceServletContextListener;
import com.google.inject.servlet.ServletModule;

import java.util.LinkedList;
import java.util.List;

/**
 * <p>A context listener to initialize create Guice injector from
 * essential's modules, the servlet module and custom modules.</p>
 *
 * <p>Extend this class and override {@link #getCustomModules()} to add
 * you own Guice modules.</p>
 *
 * <p>Override {@link #getServletModule()} to provide you own servlet
 * module (e.g. in case you want to customize Guice's servlet
 * dispatching).</p>
 *
 * <p>Override {@link #getInjector()} to manually create
 * your injector (not recommended)</p>
 *
 * @author Christian Bick
 * @since 26.12.11
 */
public class EssentialServletContextListener extends GuiceServletContextListener {

    /**
     * <p>Gets the Guice {@link Injector}.</p>
     *
     * <p>Override this method to
     * manually create your injector (not recommended).</p>
     *
     * @return The injector
     */
    @Override
    protected Injector getInjector() {
        List<Module> modules = getCustomModules();
        modules.add(new GuiceContextModule());
        modules.add(new GuiceControllerModule());
        modules.add(getServletModule());
        return Guice.createInjector(Stage.PRODUCTION, modules);
    }

    /**
     * <p>Gets the Guice {@link ServletModule} used for servlet dispatching
     * and other configurations usually configured in web.xml.</p>
     *
     * <p>Override this method to provide you own servlet
     * module (e.g. in case you want to customize Guice's servlet
     * dispatching).</p>
     *
     * @return The servlet module
     */
    protected Module getServletModule() {
        return new ServletModule() {
            @Override
            protected void configureServlets() {
                serve("/*").with(GuiceServlet.class);
            }
        };
    }

    /**
     * <p>Gets custom Guice {@link Module}s used for configuring application
     * specific injection.</p>
     *
     * <p>Override this method to add your own Guice modules.</p>
     *
     * @return The list of modules
     */
    protected List<Module> getCustomModules() {
        return new LinkedList<Module>();
    }
}
